
#include <unittest++/UnitTest++.h>

#include "VKRecorderRoot.hh"
#include "VKRunAction.hh"
#include "VKPhysicsList.hh"

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4SystemOfUnits.hh"

#include <TTree.h>
#include <TFile.h>
#include <TParameter.h>
#include <TMath.h>

#include <string> // words and sentences
#include <fstream> // Stream class to both read and write from/to files

using namespace std;



bool LookForStringInTestDir(string str) {
    system("rm -f test/dump.txt");
    system("cd test; ls > dump.txt");  // dump contents of test directory to file
    // read back in from file and check that no root file has been created
    ifstream dump("test/dump.txt");
    string line;
    bool found = false;
    if (dump.is_open()) {
        while (getline(dump, line)) {
            if (line.find(str) != string::npos) found = true;
        }
    }
    dump.close();
    system("rm test/dump.txt");  // delete temp file
    return found;
}



SUITE(VKRecorderRootTest) {

    class SetupFixture {
        public:
            SetupFixture()
            {
                // get pointer to run manager
                runManager = G4RunManager::GetRunManager();
                
                // get pointer to UI manager
                UImanager = G4UImanager::GetUIpointer();
                
                // get pointer to user action
                const VKRunAction *runAction = 
                    static_cast<const VKRunAction*>(runManager->GetUserRunAction()); 
                
                // get (dynamic) pointer to recorder
                recorder = static_cast<const VKRecorderRoot*>(runAction->GetRecorder());

                // keep only beta detector in setup
                UImanager->ApplyCommand("/VK/detector/volumes/betaDetector true");
                UImanager->ApplyCommand("/VK/detector/volumes/magnet false");
                UImanager->ApplyCommand("/VK/detector/volumes/catcherFoil false");
                UImanager->ApplyCommand("/VK/detector/volumes/shield false");

                // do *not* store trajectory data (i.e. no virtual detectors)
                UImanager->ApplyCommand("/VK/storeTrajectoryData false");

                // disable magnetic field
                UImanager->ApplyCommand("/VK/field/enable false");
                
                // prepare 1 MeV electron inside signal detector
                UImanager->ApplyCommand("/gps/direction 0 0 1");
                UImanager->ApplyCommand("/gps/position 0 0 30.5 cm");
                UImanager->ApplyCommand("/gps/particle e-");
                UImanager->ApplyCommand("/gps/energy 1 MeV");
                
                // remove any pre-existing .root files in test directory
                system("cd test; rm -f *.root");
            }
            G4UImanager *UImanager;
            G4RunManager *runManager;
            const VKRecorderRoot *recorder;
    };

    TEST_FIXTURE(SetupFixture, Sanity) {
        CHECK_EQUAL(1, 1);
    }
    
    TEST_FIXTURE(SetupFixture, TestDefaultIsNoOutputWritten) {
        UImanager -> ApplyCommand("/run/beamOn 1");
        bool found = LookForStringInTestDir(".root");
        CHECK_EQUAL(false, found); 
    }

    TEST_FIXTURE(SetupFixture, TestSwitchZeroSuppressionOnOff) {
        UImanager -> ApplyCommand("/VK/output/zeroSuppression true");        
        CHECK_EQUAL(true, recorder->GetZeroSuppressionOn()); 
        UImanager -> ApplyCommand("/VK/output/zeroSuppression false");        
        CHECK_EQUAL(false, recorder->GetZeroSuppressionOn()); 
    }

    TEST_FIXTURE(SetupFixture, TestOpenRootFile) {
        system("cd test; rm -f *.root"); // remove any pre-existing .root files in test directory        
        UImanager -> ApplyCommand("/VK/output/openFile test/one.root");
        UImanager -> ApplyCommand("/run/beamOn 1");
        bool found = LookForStringInTestDir("one.root");
        CHECK_EQUAL(true, found); 
        system("cd test; rm -f *.root"); // clean up       
    }

    TEST_FIXTURE(SetupFixture, TestOpenSeveralRootFiles) {
        system("cd test; rm -f *.root"); // remove any pre-existing .root files in test directory        
        UImanager -> ApplyCommand("/VK/output/openFile test/two.root");
        UImanager -> ApplyCommand("/run/beamOn 1");
        UImanager -> ApplyCommand("/VK/output/openFile test/three.root");
        UImanager -> ApplyCommand("/run/beamOn 1");
        bool found2 = LookForStringInTestDir("two.root");
        bool found3 = LookForStringInTestDir("three.root");
        CHECK_EQUAL(true, found2); 
        CHECK_EQUAL(true, found3); 
        system("cd test; rm -f *.root"); // clean up       
    }

    TEST_FIXTURE(SetupFixture, TestRootFileHasDetectorData) {
        system("cd test; rm -f *.root"); // remove any pre-existing .root files in test directory        
        UImanager -> ApplyCommand("/VK/output/openFile test/test.root");
        UImanager -> ApplyCommand("/run/beamOn 3");
        TFile * f0 = new TFile("test/test.root", "READ");
        TTree * t0 = (TTree*)f0->Get("Detector");
        CHECK_EQUAL(3, t0->GetEntries()); // <- Check number of events
        Float_t _e;
        t0 -> SetBranchAddress("EDepSignal", &_e);
        for (size_t i=0; i<3; i++) {
            t0 -> GetEvent(i);
            CHECK_CLOSE(1000., _e, 1e-3); // <- Check deposited energy
        }
        system("cd test; rm -f *.root"); // clean up      
        f0 -> Close(); 
    }

    TEST_FIXTURE(SetupFixture, TestRootFileHasMetaData) {
        system("cd test; rm -f *.root"); // remove any pre-existing .root files in test directory        
        UImanager -> ApplyCommand("/VK/output/openFile test/test.root");
        UImanager -> ApplyCommand("/VK/source/betaSpectrum true");
        UImanager -> ApplyCommand("/VK/source/betaEndPoint 7.00 MeV");
        UImanager -> ApplyCommand("/VK/source/betaMinEnergy 5.40 MeV");
        UImanager -> ApplyCommand("/VK/source/betaMaxEnergy 7.00 MeV");
        UImanager -> ApplyCommand("/VK/source/diskShapedSource true");
        UImanager -> ApplyCommand("/VK/source/innerDiameter 2 mm");
        UImanager -> ApplyCommand("/VK/source/outerDiameter 10 mm");
        UImanager -> ApplyCommand("/run/beamOn 9");
        TFile * f0 = new TFile("test/test.root", "READ");
        TTree * t0 = (TTree*)f0->Get("Detector");
        int N = ((TParameter<int>*)(t0->GetUserInfo()->FindObject("SIMULATED_EVENTS")))->GetVal();
        CHECK_EQUAL(9, N); // <- Check number of events
        double area = TMath::Pi()*(pow(0.5,2)-pow(0.1,2));
        double A = ((TParameter<double>*)(t0->GetUserInfo()->FindObject("SOURCE_AREA/cm2")))->GetVal();
        CHECK_CLOSE(area, A, 1e-3); // <- Check source area
        double cov = 0.07;
        double c = ((TParameter<double>*)(t0->GetUserInfo()->FindObject("SAMPLING_COVERAGE")))->GetVal();
        CHECK_CLOSE(cov, c, 1e-3); // <- Check coverage
        double norm = ((TParameter<double>*)(t0->GetUserInfo()->FindObject("NORMALIZATION")))->GetVal();
        CHECK_CLOSE(N/cov, norm, 1e-2*N/cov); // <- Check normalization
        system("cd test; rm -f *.root"); // clean up      
        f0 -> Close(); 
        UImanager -> ApplyCommand("/VK/source/betaSpectrum false");
        UImanager -> ApplyCommand("/VK/source/diskShapedSource false");
    }
}
