
#include <unittest++/UnitTest++.h>

#include "VKRecorderRoot.hh"
#include "VKRunAction.hh"

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4SystemOfUnits.hh"


SUITE(VKTransportationInBFieldTest) {

    class SetupFixture {
        public:
            SetupFixture()
            {
                // get pointer to run manager
                runManager = G4RunManager::GetRunManager();
                
                // get pointer to UI manager
                UImanager = G4UImanager::GetUIpointer();
                
                // get pointer to user action
                const VKRunAction *runAction = 
                    static_cast<const VKRunAction*>(runManager->GetUserRunAction()); 
                
                // get pointer to recorder
                recorder = static_cast<const VKRecorderRoot*>(runAction->GetRecorder());
                
                // keep only the magnet in the setup
                UImanager->ApplyCommand("/VK/detector/volumes/magnet true");
                UImanager->ApplyCommand("/VK/detector/volumes/betaDetector false");
                UImanager->ApplyCommand("/VK/detector/volumes/catcherFoil false");
                UImanager->ApplyCommand("/VK/detector/volumes/shield false");
                
                // store trajectory data
                UImanager->ApplyCommand("/VK/storeTrajectoryData true");
            }
            G4UImanager *UImanager;
            G4RunManager *runManager;
            const VKRecorderRoot *recorder;
    };

    TEST_FIXTURE(SetupFixture, Sanity) {
        CHECK_EQUAL(1, 1);
    }
    
    TEST_FIXTURE(SetupFixture, Test5MeVElectronAlongZAxisWithoutBField) {
        // disable magnetic field
        UImanager->ApplyCommand("/VK/field/enable false");
        // shoot 5 MeV electron
        // (By default particle travels along z-axis, 
        // in positive direction, starting at z = -30 cm) 
        UImanager->ApplyCommand("/gps/energy 5 MeV");
        UImanager->ApplyCommand("/gps/particle e-");
        UImanager->ApplyCommand("/run/beamOn 1");
        // check FOCAL plane
        G4double x = recorder -> GetXfp();
        G4double y = recorder -> GetYfp();
        G4double z = recorder -> GetZfp();
        G4double e = recorder -> GetEkinfp();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(299., z/mm, 1e-3);
        CHECK_CLOSE(5000., e/keV, 1e-3);        
        // check MIDWAY plane
        x = recorder -> GetXmp();
        y = recorder -> GetYmp();
        z = recorder -> GetZmp();
        e = recorder -> GetEkinmp();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(0., z/mm, 1e-3);
        CHECK_CLOSE(5000., e/keV, 1e-3); 
        // check source       
        x = recorder -> GetXso();
        y = recorder -> GetYso();
        z = recorder -> GetZso();
        e = recorder -> GetEkinso();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(-300., z/mm, 1e-3);
        CHECK_CLOSE(5000., e/keV, 1e-3); 
    }

    TEST_FIXTURE(SetupFixture, Test8MeVElectronAlongZAxisWithBField) {
        // enable magnetic field
        UImanager->ApplyCommand("/VK/field/enable enable");
        // shoot 8 MeV electron
        // (By default particle travels along z-axis, 
        // in positive direction, starting at z = -30 cm) 
        UImanager->ApplyCommand("/gps/energy 8 MeV");
        UImanager->ApplyCommand("/gps/particle e-");
        UImanager->ApplyCommand("/run/beamOn 1");
        // check FOCAL plane
        G4double x = recorder -> GetXfp();
        G4double y = recorder -> GetYfp();
        G4double z = recorder -> GetZfp();
        G4double e = recorder -> GetEkinfp();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(299., z/mm, 1e-3);
        CHECK_CLOSE(8000., e/keV, 1e-3);        
        // check MIDWAY plane
        x = recorder -> GetXmp();
        y = recorder -> GetYmp();
        z = recorder -> GetZmp();
        e = recorder -> GetEkinmp();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(0., z/mm, 1e-3);
        CHECK_CLOSE(8000., e/keV, 1e-3);        
        // check source       
        x = recorder -> GetXso();
        y = recorder -> GetYso();
        z = recorder -> GetZso();
        e = recorder -> GetEkinso();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(-300., z/mm, 1e-3);
        CHECK_CLOSE(8000., e/keV, 1e-3); 
    }

    TEST_FIXTURE(SetupFixture, Test6MeVElectronAt45DegWithBField) {
        // enable magnetic field
        UImanager->ApplyCommand("/VK/field/enable true");
        UImanager->ApplyCommand("/VK/field/maxbfield 0.45 tesla");
        // shoot 6 MeV electron at 45 degrees
        UImanager->ApplyCommand("/gps/energy 6 MeV");
        UImanager->ApplyCommand("/gps/particle e-");
        UImanager->ApplyCommand("/gps/direction 0 1 1");
        UImanager->ApplyCommand("/gps/position 0 0 -30 cm");
        UImanager->ApplyCommand("/run/beamOn 1");
        // check FOCAL plane
        G4double x = recorder -> GetXfp();
        G4double y = recorder -> GetYfp();
        G4double z = recorder -> GetZfp();
        G4double e = recorder -> GetEkinfp();
        CHECK_CLOSE( 0.5599, x/mm, 1e-3);
        CHECK_CLOSE(-0.3241, y/mm, 1e-3);
        CHECK_CLOSE(299., z/mm, 1e-3);
        CHECK_CLOSE(6000., e/keV, 1e-3);        
        // check MIDWAY plane
        x = recorder -> GetXmp();
        y = recorder -> GetYmp();
        z = recorder -> GetZmp();
        e = recorder -> GetEkinmp();
        CHECK_CLOSE(-82.38, x/mm, 1e-2);
        CHECK_CLOSE(-42.75, y/mm, 1e-2);
        CHECK_CLOSE(0., z/mm, 1e-3);
        CHECK_CLOSE(6000., e/keV, 1e-3);        
        // check source       
        x = recorder -> GetXso();
        y = recorder -> GetYso();
        z = recorder -> GetZso();
        e = recorder -> GetEkinso();
        CHECK_CLOSE(0., x/mm, 1e-3);
        CHECK_CLOSE(0., y/mm, 1e-3);
        CHECK_CLOSE(-300., z/mm, 1e-3);
        CHECK_CLOSE(6000., e/keV, 1e-3); 
    }

}
